import controlP5.*; //<>//
ControlP5 cp5;

//Slider alphaSlider,timeRangeSlider,showPeriodSlider,speedSlider,zoomRatioSlider;
Table dataTable;
int rowNum, colNum;
float dataMatrix[][];
StringList photoIDs;
float maxExposure, minExposure;
ArrayList<ImageInfo> ImageInfos;
ArrayList<PImage> imgsAtMouse;

PImage worldMap;

float universeTime;

color night, day;
PImage colorGradient;
PFont timeFont;
float verticalLocation, barWidth;

int viewMode;
float speed, hourRangeToShow, hourPeriodToShow, zoomRatio, alphaValue;
boolean stopTime, showBackground;

Button rectModeButton, spotModeButton, mode1Button, mode2Button, mode3Button, backgroundButton, resetButton, stopButton;
boolean rectOrSpot;
Slider speedSlider, hourRangeSlider, hourPeriodSlider, zoomRatioSlider, alphaValueSlider;
Knob universeTimeKnob;

void setup() {
  fullScreen(P3D);
  //size(960, 600, P3D);

  cp5 = new ControlP5(this);

  universeTime = 0;
  speed = 100;

  worldMap = loadImage("worldmap.jpg");
  worldMap.resize(width, height);

  night = #1966FF;
  day = #FFBE00;
  colorGradient = loadImage("timeColor.png");
  timeFont = loadFont("HelveticaNeue-CondensedBold-100.vlw");

  viewMode = 1;
  hourRangeToShow = 0.2;
  hourPeriodToShow = 1;
  zoomRatio = 4;
  alphaValue = 80;
  stopTime = false;
  rectOrSpot = true;
  showBackground = true;


  // Data parse
  //
  dataTable = loadTable("tableNew.csv", "header");
  rowNum = dataTable.getRowCount();
  colNum = dataTable.getColumnCount();

  dataMatrix = new float[rowNum][9];
  photoIDs = new StringList();

  maxExposure = -100;
  minExposure = 100;

  ImageInfos = new ArrayList<ImageInfo>();
  imgsAtMouse = new ArrayList<PImage>();

  for (int i = 0; i<rowNum; i++) {
    TableRow row = dataTable.getRow(i);
    if (row.getFloat("Time")>1) {
      dataMatrix[i][0] = row.getInt("Time");

      String exposureString = row.getString("ExposureTime");
      String apertureString = row.getString("Aperture");
      String isoString = row.getString("ISO");
      float exposureTime = 0;
      float aperture = 0;
      float iso = 0;
      if (exposureString!= "0" & !exposureString.equals("Bulb")) {
        if (exposureString.length()>1 && exposureString.charAt(1)=='/') {
          exposureTime = 1/parseFloat(exposureString.substring(2));
        } else exposureTime = row.getFloat("ExposureTime");
      }

      if (apertureString.length() > 1 && isoString.length()>1) {
        aperture = row.getFloat("Aperture");
        iso = row.getFloat("ISO");
      }
      if (exposureTime!=0 & aperture!=0 & iso!=0 & !Float.isNaN(exposureTime) & !Float.isNaN(aperture) & !Float.isNaN(iso)) {
        //dataMatrix[i][1] = log(exposureTime/aperture/aperture*iso/100);
        dataMatrix[i][1] = log(aperture*aperture/exposureTime) - log(iso/100.f);
        if (dataMatrix[i][1] > maxExposure) maxExposure = dataMatrix[i][1];
        if (dataMatrix[i][1] < minExposure) minExposure = dataMatrix[i][1];
        if (Float.isNaN(dataMatrix[i][1])) {
          println(iso);
          println(aperture);
          println(exposureTime);
          println(" ");
        }
      }

      dataMatrix[i][2] = row.getFloat("Lon");
      dataMatrix[i][3] = row.getFloat("Lat");

      String focalLengthString = row.getString("FocalLength");
      float focalLength = 0;
      if (focalLengthString.length() >1) {
        focalLength = parseFloat(focalLengthString.substring(0, focalLengthString.length()-3));
        dataMatrix[i][4] = 2*atan((43.27/2)/focalLength);
      }

      dataMatrix[i][5] = row.getFloat("Red");
      dataMatrix[i][6] = row.getFloat("Green");
      dataMatrix[i][7] = row.getFloat("Blue");
      dataMatrix[i][8] = row.getFloat("ImgDic");
      //dataMatrix[i][9] = row.getFloat("id");
      photoIDs.append(row.getString("id"));
    }
  }
  // Create Objects
  for (int i = 0; i < rowNum; i++) {
    if (dataMatrix[i][1]!=0&&dataMatrix[i][0]!=0&&dataMatrix[i][2]!=0&&dataMatrix[i][3]!=0&&dataMatrix[i][4]!=0) {
      //println("::" +dataMatrix[i][0]+ ":: ::"+dataMatrix[i][1] + ":: ::"+ dataMatrix[i][2]+ ":: ::"+ dataMatrix[i][3]+ ":: ::"+dataMatrix[i][4] + "::");
      String imgDirectory = "data/img/2015-07-"+str(int(dataMatrix[i][8]/100))+"/"+str(int(dataMatrix[i][8])%100)+"/"+photoIDs.get(i)+".jpg";
      //if (i<3) println(imgDirectory);
      //thumbnail = loadImage(imgDirectory);
      //if (thumbnail != null) {
      //println(imgDirectory);

      ImageInfos.add(new ImageInfo(dataMatrix[i][0], 
        dataMatrix[i][1], 
        dataMatrix[i][2], 
        dataMatrix[i][3], 
        dataMatrix[i][4], 
        dataMatrix[i][5], 
        dataMatrix[i][6], 
        dataMatrix[i][7], 
        imgDirectory));
      //}
    }
  }

  //blendMode(ADD);
  noStroke();
  smooth();
}

void draw() {
  background(0);
  imgsAtMouse = new ArrayList<PImage>();
  if (showBackground) 
    image(worldMap, 0, 0);

  //if (mousePressed) 
  //  universeTime = map(mouseX, 0, width, 3600*24, 0);
  //else 



  float zeroZoneTime =  universeTime *3600;
  for (ImageInfo i : ImageInfos) {
    float hourDiff = map(i.longitude, -180, 180, -12, 12);
    if (abs(zeroZoneTime - i.time + hourDiff*3600) < hourRangeToShow*3600)
      i.drawInfo(zeroZoneTime);

    //if (!showBackground) {
    String img = i.showThumb(mouseX, mouseY, zeroZoneTime);
    //if (img != null && imgsAtMouse.size()<9) {
    if (!img.equals("")) {
      //imgsAtMouse.add(img);
      println("ImageAdded");
      PImage imgatmouse = loadImage(img);
      //imgsAtMouse.add(loadImage(img));
      try {
        image(imgatmouse, mouseX, mouseY, imgatmouse.width*2, imgatmouse.height*2);
      }catch( NullPointerException e){
        println("error found image");
      }
    }

    //println(imgsAtMouse.size());
    //for (int j = 0; j<imgsAtMouse.size(); j++) {
    //  float thumbDisplaySize = 10;
    //  image(imgsAtMouse.get(j), mouseX + (j%3)*thumbDisplaySize, mouseY + (j/3)*thumbDisplaySize, thumbDisplaySize, thumbDisplaySize);
    //}

    //imgsAtMouse.clear();
    //}
  }

  drawTimeBar();
  gui();
  if (!stopTime && !universeTimeKnob.isMousePressed()) {
    universeTime += speed/3600;
    if (universeTime > 24) universeTime = 0;
  }
}


void drawTimeBar() {
  verticalLocation = 0.75*height;
  barWidth = height/100;

  float currentZonetime;
  textFont(timeFont, int(36.f/1200*height));
  textAlign(CENTER, BOTTOM);
  //currentZonetime = (universeTime)%24;
  //fill(colorGradient.get(int(map(currentZonetime, 0, 24, 0, 1440)), 5), 240);
  //text(str(int(universeTime)%24), width/2, verticalLocation);

  //currentZonetime = (universeTime+8)%24;
  //fill(colorGradient.get(int(map(currentZonetime, 0, 24, 0, 1440)), 5), 240);
  //text(str(int(universeTime+8)%24), width*20/24, verticalLocation);

  //currentZonetime = (universeTime+16)%24;
  //fill(colorGradient.get(int(map(currentZonetime, 0, 24, 0, 1440)), 5), 240);
  //text(str(int(universeTime+16)%24), width*4/24, verticalLocation);

  float zeroZoneTime = (universeTime)%24;
  float bottomBoundry = verticalLocation + barWidth/2;
  float topBoundry = verticalLocation - barWidth/2;
  textureMode(NORMAL);
  noStroke();
  if (zeroZoneTime <12) {
    beginShape();
    texture(colorGradient);
    vertex(0, topBoundry, (zeroZoneTime+12)/24, 0.5);
    vertex(width*(12-zeroZoneTime)/24, topBoundry, 1, 0.5);
    vertex(width*(12-zeroZoneTime)/24, bottomBoundry, 1, 0.5);
    vertex(0, bottomBoundry, (zeroZoneTime+12)/24, 0.5);
    endShape();

    beginShape();
    texture(colorGradient);
    vertex(width*(12-zeroZoneTime)/24, topBoundry, 0, 0.5);
    vertex(width, topBoundry, (zeroZoneTime+12)/24, 0.5);
    vertex(width, bottomBoundry, (zeroZoneTime+12)/24, 0.5);
    vertex(width*(12-zeroZoneTime)/24, bottomBoundry, 0, 0.5);
    endShape();
  } else {
    beginShape();
    texture(colorGradient);
    vertex(0, topBoundry, (zeroZoneTime-12)/24, 0.5);
    vertex(width*(36-zeroZoneTime)/24, topBoundry, 1, 0.5);
    vertex(width*(36-zeroZoneTime)/24, bottomBoundry, 1, 0.5);
    vertex(0, bottomBoundry, (zeroZoneTime-12)/24, 0.5);
    endShape();

    beginShape();
    texture(colorGradient);
    vertex(width*(36-zeroZoneTime)/24, topBoundry, 0, 0.5);
    vertex(width, topBoundry, (zeroZoneTime-12)/24, 0.5);
    vertex(width, bottomBoundry, (zeroZoneTime-12)/24, 0.5);
    vertex(width*(36-zeroZoneTime)/24, bottomBoundry, 0, 0.5);
    endShape();
  }

  float sunSetLocation = width*(7-universeTime)/24.f;
  float sunRiseLocation = width*(19-universeTime)/24.f;
  if (sunSetLocation < 0) sunSetLocation += width;
  if (sunRiseLocation < 0) sunRiseLocation += width;
  strokeWeight(1);
  stroke(20);
  line(sunSetLocation, 0, sunSetLocation, height);
  line(sunRiseLocation, 0, sunRiseLocation, height);
  noStroke();
}

void keyPressed() {
  if (key == '1') {
    speed +=10;
  }
  if (key == '2') {
    speed -= 10;
  }
  if (key == '3') {
    hourPeriodToShow +=0.1;
  }
  if (key == '4') {
    hourPeriodToShow -= 0.1;
  }
  if (key == '5') {
    hourRangeToShow +=0.1;
  }
  if (key == '6') {
    if (hourRangeToShow>0.1) hourRangeToShow -= 0.1;
  }
  if (key == 's' || key == 'S') {
    stopTime = !stopTime;
  }
  if (key == 'm' || key == 'M') {
    viewMode = 3-viewMode;
  }
}

void mousePressed() {
  if (rectModeButton.mouseOnButton()) {
    rectModeButton.status = true;
    spotModeButton.status = false;
    rectOrSpot = true;
  }

  if (spotModeButton.mouseOnButton()) {
    spotModeButton.status = true;
    rectModeButton.status = false;
    rectOrSpot = false;
  }

  if (mode1Button.mouseOnButton()) {
    mode1Button.status = true;
    mode2Button.status = false;
    mode3Button.status = false;
    viewMode = 1;
  }
  if (mode2Button.mouseOnButton()) {
    mode2Button.status = true;
    mode1Button.status = false;
    mode3Button.status = false;
    viewMode = 2;
  }
  if (mode3Button.mouseOnButton()) {
    mode3Button.status = true;
    mode2Button.status = false;
    mode1Button.status = false;
    viewMode = 3;
  }
  if (backgroundButton.mouseOnButton()) {
    showBackground = !showBackground;
    backgroundButton.status = !backgroundButton.status;
  }
  if (stopButton.mouseOnButton()) {
    stopTime = !stopTime;
    stopButton.status = !stopButton.status;
  }
  if (resetButton.mouseOnButton()) {
    hourRangeSlider.setValue(0.2);
    hourPeriodSlider.setValue(1);
    zoomRatioSlider.setValue(4);
    alphaValueSlider.setValue(80);
    speedSlider.setValue(100);

    stopTime = false;
    stopButton.status = false;

    viewMode = 1;
    mode1Button.status = true;
    mode2Button.status = false;
    mode3Button.status = false;

    rectOrSpot = true;
    rectModeButton.status = true;
    spotModeButton.status = false;

    showBackground = true;
    backgroundButton.status = true;
  }
}